% MatlabSampleRecon1
%
% Script to illustrate use of Matlab for reconstruction of Feb-Sep mean temperature from PCs 
% of four Poland site chronologies. Stopping step for stepwise regression of final model is
% selected by cross-validation. For this trial data, with no lags, cross-validation is leave-1-out.
%
% If you want to use split-sample validation to select the final step for the regression, see
% the sister script MatlabSampleRecon2.
%
% In general, I reconstruct by writing a script such as this one, which then calls functions
% to do blocks of work.  Some of the fuctions are "built in" Matlab functions. Others are 
% "user-written", or functions I have written over the years to do specific repetitive tasks.
%
% This script has following steps:
%
% -Reads in the monthly T data and call Meko Matlab function seaspt to seasonalize to
%   Feb-Sept average
%- Reads in a time series matrix (tsm) of four residual site chronologies: 
%   1) WEG Abies alba, 2) Robert's oak, 3) Picea (Pola022), and 4) another Abies Alba
%   chronology (Pola024)
%- Calls Meko Matlab function eigen1 for PCA on the chronologies and to produce the
%   four time series PC1, PC2, PC3 and PC4
%- Specify calibration period 1940-2004 (Group II choose this)
%- Calls Meko Matlab function reconsw1 to do stepwise regression of y on x1, x2,
%   x3, x4 and generate a reconstruction, yhat, covering 1836-2004
%- Makes a few figure windows, including one with a time plot ot the
%   reconstruction and its 50% confidence interval
%
% The results here may not exactly match those of Group II got because of dfferences in
% versions of the chronologies used. If you want to try to duplicate the Group II reconstruction, 
% you will need to make sure the input data is identical to that used by Group II. 
%
% At last, you will see 8 figure windows.  Windows 1-5 are for analysis of
% residuals. Those windows are generated by function reschk2, called by function reconsw1.
% Remaining plots are made by this script. A list of the figure windows:
%
% 1 Time plots of observed (yo) and reconstructed (yh) predicand (top), and
% of the regression residuals (yo-yh) (bottom). A trend line is fit in the
% bottom, with a test for significance of linear trend in residuals.
% 
% 2 Histogram of regression residuals (like this to look normal)
%
% 3 Autocorrelation function of regression residuals
%
% 4 Scatterplot of residuals at time t against residuals at time t-1, with
% ftted line. This is another way of looking a lag-1 persistence in
% residuals
%
% 5  Scatterplot of residuals against fitted values (predicted values of
% predictand). This plot used to judge mainly whether any pattern of
% dependence of variance of residuals on the predictions. For example, a
% fanning out left to right could indicate variance of residuals not
% constant, but a function of sized of fitted values.
%
% 6 Time plots of the four original input chronologies before conversion by
% PCA. This plot can be used to check that no mistakes in reading in the
% chronologies
%
% 7 Bar chart of percentage of tree-ring variance in each PC.
%
% 8 Time plot of the reconstruction with its 50% confidence interval
%
% After running this script, you will also see a structure variable "Result" in the workspace. 
% Result has output statistics and time series (e.g., the reconstruction)
% in various fields. Read the opening comments on "Output" at the start of
% function reconsw1 for definitions of the fields.


clear all
close all
clc


%--- HARD CODE

%---- READ IN THE MONTHLY T DATA AND SEASONALIZE TO FEB-SEPT

% Use Matlab's "load" function, which works with ascii txt data
flnm = 'tempfrom1929.dat';
D=load(flnm,'-ascii'); % D is 13-col, with year and Jan-dec data in deg C

% set up input to call seaspt
begmo=2; endmo=9;  % Feb and Sept as start and end month
k = 2; % option specifying the seasonalizing is by average rather than total
F = seaspt(D,begmo,endmo,k); % year as col 1, Feb-Sep T as col 2

% Let y1 be the temperature series, and yry1 be the corresponding vector of
% years
yry1 = F(:,1);
y1 = F(:,2);


%---- READ IN THE RESIDUAL SITE CHRONOLOGIES
%
% Here, will use function tblread, which reads tabular data assumed to have
% case names in column 1 (these are the years, and variable names in row 1
% above the other columns

flnm = 'Res_chronologies.txt'; % this is the input file
[X1, Xnms, CaseNames] = tblread(flnm,'tab'); % X1 is 4-col matrix of the chronologies

% convert the CaseNames from string to number-- this will be column vector of
% years
yrX1 = str2num(CaseNames); % col vector of years

% Xnms is a character matrix with site ids in each row; convert those to a
% cell variable, with each member an id
nms = cellstr(Xnms);

% Make a quick plot of the chronologies
xlims=[min(yrX1) max(yrX1)]; % for setting the limits of x axis



%---- PCA ON THE SITE CHRONOLOGIES
%
% Here, will use my function eigen1, which calls various built-in matlab
% functions. We will do the PCA on the correlation matrix rather than the
% covariance matrix of the chronologies

k = 1; % specify use the correlation matrix for the PCA
[R,V,L,S,G,B]=eigen1(X1,k); % returns correlation matrix in R, loadings of PCs in V, eigenvalues in L,
%  and time series of PC scores (PC1, PC2, PC3, PC4) in G
yrG = yrX1;

% From the eigenvalues, compute the percentage of variance in each PC
p = 100*diag(L)/sum(diag(L));




%-- REGRESSION AND RECONSTRUCTION 
%
% Done with call to reconsw1 (reconstruction stepwise 1). Set up the input
% arguments for call to reconsw1.

%  Set up X and y, the time series for the overlap, defined by default as the
%  calibration period, but wanted by us 1940-2004
yrgo =1940;
yrsp =2004;


% specs = structure of specifications
specs.pin=0.05; %   .pin for entry in stepwise (see notes for special case)
specs.pout=0.10; %  for removal in stepwise (see notes for special case)
specs.nlag=0; %   .nlag: maximum negative lag on predictors (
specs.plag=0; %   .plag: maximum positive lag allowed on predictors
specs.CalYrs = [yrgo yrsp] ; %   .CalYrs (1x2)i or []:   a specified first and last year (on y) of
%      calibration period; if [], function will use longest possible calib  period
specs.LRA= [1 1]; %  : analysis of residual (1x2)L
%       (1) true==do it;   false=skip it  (1 is true, 0 is false)
%       (2) true==do graphics (slower); false=skip graphics [this option
%       meaningless if LRA(1) true
specs.pthresh=1;  % (1x1)r threshold decimal proportion (e.g., 0.95) of
%       cross-validationa model that must reach whatever step is selected for
%        final model

% SpecsSim: specifications for call to pdgmsim from reconsw1
SpecsSim.nsim=1000; % (1x1)i number of simulations (e.g., 1000)
SpecsSim.mtaper=0.10; %  (1x1)r  decimal fractional total of series to be tapered (e.g.,
%        0.10, which means 5% on front and 5% on back
SpecsSim.kopt=[1 2]; % (1x2)i options
%       (1) force sims to have same mean and standard deviation as the
%               observed series
%           ==1 yes;  ==2 no
%       (2)  turn off the msgbox warning about failing Lilliefors
%           ==1 yes;  ==2 no (default if length(kopt)==1)
SpecsSim.len=[]; %  (1x1)i  or []. desired length of simulations; or, if [], same
%           length as the observed series

%  Call to reconsw1 to do stepwise reconstruct
%Result=reconsw1(X,yrX,y,yry,specs,SpecsSim);
Result=reconsw1(G,yrG,y1,yry1,specs,SpecsSim);



%---- COMPUTE 50% CONFIDENCE INTERVAL FOR RECONSTUCTION USING
% CROSS-VALIDATION RESIDUALS. ASSUMING RESIDUALS NORMALLY DISTRIBUTED, THIS
% INTERVAL IS   yhat +- 0.67449 RMSEv, where yhat is the reconstructed
% temperature, and RMSEv is the root-mean-square error of cross-validation
% residuals

yh=Result.yh; % reconstructed Feb-Sep temperature
yryh = Result.yryh; % year vector for yh
c = Result.statsv.RMSE ; % cross-validation RMSE

yhhi = yh + 0.67449*c; % upper confidence interval
yhlo = yh - 0.67449*c; % lower confidence interval


%---- MISCELLANEOUS FIGURES 

%-----  Time plots of original chronologies for their common period
figure(6);
h = plot(yrX1,X1);
set(gca,'XLim',xlims) ; % set x-axis to start and end at data limits of years
legend(nms)
xlabel('Year')
ylabel('Index')
title('Residual site chronologies')


%----- Bar chart of percentage of tree-ring variance explained by PCs
figure(7)
h = bar(p);
xlabel ('PC#')
ylabel('% variance explained')
title('Tree-ring percentage variance explained by each PC')

%---- Time plot or reconstruction with 50% confidence interval
figure (8)
[cL,cB,cW,cH]=figsize(.8,.6);
set(gcf,'Position',[cL cB cW cH]);

stryrs=[num2str(yryh(1)) '-'  num2str(yryh(end))]; % make string for labeling years in title

xlims =[yryh(1) yryh(end)]; % want as x-axis  limits on plot
h = plot(yryh,yh,yryh, yhlo, yryh,yhhi,[yryh(1) yryh(end)],[mean(yh) mean(yh)]); % make the plot
set(h(1),'color',[0 0 1]);
set(h(2),'Color',[1 0 0]);
set(h(3),'Color',[1 0 0]);
set(h(4),'Color',[0 0 0]);
set(gca,'XLim',xlims); % set x-axis limits

xlabel('Year');
ylabel('T (deg C)')
title(['Reconstructed T, ' stryrs ', with 50% confidence interval'])


%---- MESSAGE ABOUT HOW TO ACCESS DATA AND STATISTICS

str_msg1 = {['Structure variable ''Results'' has data and statistics for the regression and reconstruction'],...
    'You can access this data from the Matlab command prompt by typing ''Result''',...
    'For example, Result.yh and Result.yryh are the reconstruction and its year vector.',...
    'Fields Result.statsc and Results.statsv hold the statistics for calibration and validation.',...
    '     For example, type ''Result.statsc'' and you will see the regression R2 and other statistics.',...
    '     Type ''Result.statsv'' and you will see RE and other validation statistics'};
 uiwait(msgbox(str_msg1,'Message','modal'));
